@@ -19,6 +19,10 @@
  *
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include "e-util/e-util-private.h"
 
 #include "e-cal-shell-view-private.h"
@@ -92,7 +96,7 @@ cal_shell_view_date_navigator_selection_changed_cb (ECalShellView *cal_shell_vie
 	icaltimezone *timezone;
 	time_t start, end, new_time;
 	gboolean starts_on_week_start_day;
-	gint new_days_shown;
+	gint new_days_shown, old_days_shown;
 	gint week_start_day;
 
 	cal_shell_content = cal_shell_view->priv->cal_shell_content;
@@ -136,6 +140,9 @@ cal_shell_view_date_navigator_selection_changed_cb (ECalShellView *cal_shell_vie
 		g_date_compare (&end_date, &new_end_date) == 0)
 		return;
 
+	old_days_shown =
+		g_date_get_julian (&end_date) -
+		g_date_get_julian (&start_date) + 1;
 	new_days_shown =
 		g_date_get_julian (&new_end_date) -
 		g_date_get_julian (&new_start_date) + 1;
@@ -156,7 +163,10 @@ cal_shell_view_date_navigator_selection_changed_cb (ECalShellView *cal_shell_vie
 
 	/* Switch views as appropriate, and change the number of
 	 * days or weeks shown. */
-	if (new_days_shown > 9) {
+	if (view_type == GNOME_CAL_WORK_WEEK_VIEW && old_days_shown == new_days_shown) {
+		/* keep the work week view when has same days shown */
+		switch_to = GNOME_CAL_WORK_WEEK_VIEW;
+	} else if (new_days_shown > 9) {
 		if (view_type != GNOME_CAL_LIST_VIEW) {
 			ECalendarView *calendar_view;
 
@@ -273,7 +283,7 @@ cal_shell_view_selector_popup_event_cb (EShellView *shell_view,
 
 static void
 cal_shell_view_selector_client_added_cb (ECalShellView *cal_shell_view,
-                                         ECal *client)
+                                         ECalClient *client)
 {
 	ECalShellContent *cal_shell_content;
 	GnomeCalendar *calendar;
@@ -290,7 +300,7 @@ cal_shell_view_selector_client_added_cb (ECalShellView *cal_shell_view,
 
 static void
 cal_shell_view_selector_client_removed_cb (ECalShellView *cal_shell_view,
-                                           ECal *client)
+                                           ECalClient *client)
 {
 	ECalShellContent *cal_shell_content;
 	GnomeCalendar *calendar;
@@ -331,12 +341,12 @@ cal_shell_view_user_created_cb (ECalShellView *cal_shell_view,
 {
 	ECalShellSidebar *cal_shell_sidebar;
 	ECalModel *model;
-	ECal *client;
+	ECalClient *client;
 	ESource *source;
 
 	model = e_calendar_view_get_model (calendar_view);
 	client = e_cal_model_get_default_client (model);
-	source = e_cal_get_source (client);
+	source = e_client_get_source (E_CLIENT (client));
 
 	cal_shell_sidebar = cal_shell_view->priv->cal_shell_sidebar;
 	e_cal_shell_sidebar_add_source (cal_shell_sidebar, source);
@@ -532,8 +542,10 @@ e_cal_shell_view_private_constructed (ECalShellView *cal_shell_view)
 
 	/* Give GnomeCalendar a handle to the date navigator, memo and task table. */
 	gnome_calendar_set_date_navigator (calendar, date_navigator);
-	gnome_calendar_set_memo_table (calendar, memo_table ? GTK_WIDGET (memo_table) : NULL);
-	gnome_calendar_set_task_table (calendar, task_table ? GTK_WIDGET (task_table) : NULL);
+	gnome_calendar_set_memo_table (
+		calendar, memo_table ? GTK_WIDGET (memo_table) : NULL);
+	gnome_calendar_set_task_table (
+		calendar, task_table ? GTK_WIDGET (task_table) : NULL);
 
 	e_calendar_item_set_get_time_callback (
 		date_navigator->calitem, (ECalendarItemGetTimeCallback)
@@ -675,12 +687,9 @@ e_cal_shell_view_private_dispose (ECalShellView *cal_shell_view)
 	ECalShellViewPrivate *priv = cal_shell_view->priv;
 	gint i;
 
-	/* Calling calendar's save state from here, because it is too late in its dispose */
-	if (priv->cal_shell_content)
-		e_cal_shell_content_save_state (priv->cal_shell_content);
-
-	/* Calling calendar's save state from here, because it is too late in its dispose */
-	if (priv->cal_shell_content)
+	/* Calling calendar's save state from here,
+	 * because it is too late in its dispose. */
+	if (priv->cal_shell_content != NULL)
 		e_cal_shell_content_save_state (priv->cal_shell_content);
 
 	DISPOSE (priv->cal_shell_backend);
@@ -812,35 +821,10 @@ e_cal_shell_view_set_status_message (ECalShellView *cal_shell_view,
 	cal_shell_view->priv->calendar_activity = activity;
 }
 
-struct ForeachTzidData
-{
-	ECal *source_client;
-	ECal *dest_client;
-};
-
-static void
-add_timezone_to_cal_cb (icalparameter *param, gpointer data)
-{
-	struct ForeachTzidData *ftd = data;
-	icaltimezone *tz = NULL;
-	const gchar *tzid;
-
-	g_return_if_fail (ftd != NULL);
-	g_return_if_fail (ftd->source_client != NULL);
-	g_return_if_fail (ftd->dest_client != NULL);
-
-	tzid = icalparameter_get_tzid (param);
-	if (!tzid || !*tzid)
-		return;
-
-	if (e_cal_get_timezone (ftd->source_client, tzid, &tz, NULL) && tz)
-		e_cal_add_timezone (ftd->dest_client, tz, NULL);
-}
-
 void
 e_cal_shell_view_transfer_item_to (ECalShellView *cal_shell_view,
                                    ECalendarViewEvent *event,
-                                   ECal *destination_client,
+                                   ECalClient *destination_client,
                                    gboolean remove)
 {
 	icalcomponent *icalcomp;
@@ -854,7 +838,7 @@ e_cal_shell_view_transfer_item_to (ECalShellView *cal_shell_view,
 
 	g_return_if_fail (E_IS_CAL_SHELL_VIEW (cal_shell_view));
 	g_return_if_fail (event != NULL);
-	g_return_if_fail (E_IS_CAL (destination_client));
+	g_return_if_fail (E_IS_CAL_CLIENT (destination_client));
 
 	if (!is_comp_data_valid (event))
 		return;
@@ -864,34 +848,30 @@ e_cal_shell_view_transfer_item_to (ECalShellView *cal_shell_view,
 
 	/* Put the new object into the destination calendar. */
 
-	success = e_cal_get_object (
-		destination_client, uid, NULL, &icalcomp, NULL);
+	success = e_cal_client_get_object_sync (
+		destination_client, uid, NULL, &icalcomp, NULL, NULL);
 
 	if (success) {
 		icalcomponent_free (icalcomp);
-		success = e_cal_modify_object (
+		success = e_cal_client_modify_object_sync (
 			destination_client, icalcomp_event,
-			CALOBJ_MOD_ALL, NULL);
+			CALOBJ_MOD_ALL, NULL, NULL);
 
 		/* do not delete the event when it was found in the calendar */
 		return;
 	} else {
 		icalproperty *icalprop;
 		gchar *new_uid;
-		struct ForeachTzidData ftd;
-
-		ftd.source_client = event->comp_data->client;
-		ftd.dest_client = destination_client;
+		GError *error = NULL;
 
 		if (e_cal_util_component_is_instance (icalcomp_event)) {
-			success = e_cal_get_object (
+			success = e_cal_client_get_object_sync (
 				event->comp_data->client,
-				uid, NULL, &icalcomp, NULL);
+				uid, NULL, &icalcomp, NULL, NULL);
 			if (success) {
 				/* Use master object when working
 				 * with a recurring event ... */
-				icalcomp_clone =
-					icalcomponent_new_clone (icalcomp);
+				icalcomp_clone = icalcomponent_new_clone (icalcomp);
 				icalcomponent_free (icalcomp);
 			} else {
 				/* ... or remove the recurrence ID ... */
@@ -924,11 +904,11 @@ e_cal_shell_view_transfer_item_to (ECalShellView *cal_shell_view,
 		}
 
 		new_uid = NULL;
-		icalcomponent_foreach_tzid (icalcomp_clone, add_timezone_to_cal_cb, &ftd);
-		success = e_cal_create_object (
-			destination_client, icalcomp_clone, &new_uid, NULL);
+		success = e_cal_client_create_object_sync (
+			destination_client, icalcomp_clone, &new_uid, NULL, &error);
 		if (!success) {
 			icalcomponent_free (icalcomp_clone);
+			g_debug ("%s: Failed to create object: %s", G_STRFUNC, error ? error->message : "Unknown error");
 			return;
 		}
 
@@ -937,7 +917,7 @@ e_cal_shell_view_transfer_item_to (ECalShellView *cal_shell_view,
 	}
 
 	if (remove) {
-		ECal *source_client = event->comp_data->client;
+		ECalClient *source_client = event->comp_data->client;
 
 		/* Remove the item from the source calendar. */
 		if (e_cal_util_component_is_instance (icalcomp_event) ||
@@ -951,11 +931,11 @@ e_cal_shell_view_transfer_item_to (ECalShellView *cal_shell_view,
 				rid = icaltime_as_ical_string_r (icaltime);
 			else
 				rid = NULL;
-			e_cal_remove_object_with_mod (
-				source_client, uid, rid, CALOBJ_MOD_ALL, NULL);
+			e_cal_client_remove_object_sync (
+				source_client, uid, rid, CALOBJ_MOD_ALL, NULL, NULL);
 			g_free (rid);
 		} else
-			e_cal_remove_object (source_client, uid, NULL);
+			e_cal_client_remove_object_sync (source_client, uid, NULL, CALOBJ_MOD_THIS, NULL, NULL);
 	}
 }
 
@@ -1112,10 +1092,10 @@ e_cal_shell_view_update_timezone (ECalShellView *cal_shell_view)
 	clients = e_cal_shell_sidebar_get_clients (cal_shell_sidebar);
 
 	for (iter = clients; iter != NULL; iter = iter->next) {
-		ECal *client = iter->data;
+		ECalClient *client = iter->data;
 
-		if (e_cal_get_load_state (client) == E_CAL_LOAD_LOADED)
-			e_cal_set_default_timezone (client, timezone, NULL);
+		if (e_client_is_opened (E_CLIENT (client)))
+			e_cal_client_set_default_timezone (client, timezone);
 	}
 
 	g_list_free (clients);
