@@ -42,7 +42,7 @@
 #include <libedataserver/e-url.h>
 #include <libedataserver/e-account-list.h>
 #include <libedataserver/e-proxy.h>
-#include <libecal/e-cal.h>
+#include <libecal/e-cal-client.h>
 #include <libedataserverui/e-cell-renderer-color.h>
 #include <libedataserverui/e-passwords.h>
 
@@ -71,9 +71,12 @@ static void
 ensure_google_source_group (void)
 {
 	ESourceList  *slist;
+	GError *error = NULL;
 
-	if (!e_cal_get_sources (&slist, E_CAL_SOURCE_TYPE_EVENT, NULL)) {
-		g_warning ("Could not get calendar source list from GConf!");
+	if (!e_cal_client_get_sources (&slist, E_CAL_CLIENT_SOURCE_TYPE_EVENTS, &error)) {
+		g_debug ("%s: Could not get calendar source list: %s", G_STRFUNC, error ? error->message : "Unknown error");
+		if (error)
+			g_error_free (error);
 		return;
 	}
 
@@ -415,6 +418,9 @@ static void
 retrieve_list_clicked (GtkButton *button, GtkComboBox *combo)
 {
 	ESource *source;
+	#ifdef HAVE_LIBGDATA_0_9
+	GDataClientLoginAuthorizer *authorizer;
+	#endif
 	GDataCalendarService *service;
 	GDataFeed *feed;
 	gchar *user, *password, *tmp;
@@ -445,8 +451,14 @@ retrieve_list_clicked (GtkButton *button, GtkComboBox *combo)
 		return;
 	}
 
+	#ifdef HAVE_LIBGDATA_0_9
+	authorizer = gdata_client_login_authorizer_new ("evolution-client-0.1.0", GDATA_TYPE_CALENDAR_SERVICE);
+	service = gdata_calendar_service_new (GDATA_AUTHORIZER (authorizer));
+	if (!gdata_client_login_authorizer_authenticate (authorizer, user, password, NULL, &error)) {
+	#else
 	service = gdata_calendar_service_new ("evolution-client-0.1.0");
 	if (!gdata_service_authenticate (GDATA_SERVICE (service), user, password, NULL, &error)) {
+	#endif
 		/* Error! */
 		claim_error (parent, error->message);
 		g_error_free (error);
@@ -556,6 +568,9 @@ retrieve_list_clicked (GtkButton *button, GtkComboBox *combo)
 	}
 
 	g_object_unref (service);
+	#ifdef HAVE_LIBGDATA_0_9
+	g_object_unref (authorizer);
+	#endif
 	g_free (user);
 }
 
